function GenerateWorld(indices)
% Generates the structure World (boundaries of obstacles) from UObstacles (uninflated obstacles)
global UObstacles UBoundary
global World

% UObstacles -- a structure defining various shapes
%           .N       -- number of obstacles
%           .uobs{a} -- cell array of obstacles
%
%                   .CreatedFromPoints -- boolian describing whether it is created from scan points
%                   .points -- (optional - required only if .CreatedFromPoints is 1) the coordinates of points
%                   .Geometry
%
%                            .type = 'circle'
%                            .r -- radius of circle
%                            .c -- center of circle
%
%                            .type = 'straight_rectangle'
%                            .xrange -- a vector [xMin, xMax]
%                            .yrange -- a vector [yMin, yMax]
%
%                            .type = 'polygon'
%                            .vertices -- 2xN matrix of coordinates

% Structure World
%                .N                 -- Number of elements in the world
%                .Center(:,k)     -- A central point of the k-th element
%                .RMax(k)       -- An upperbound on the 'radius' of the k-th element
%                .element{k}        -- A structure defining a line/curve
%
%
%                           .type       =  'LineSegment'
%                           .ends       -- A 2x2 matrix each each column representing a vertex
%                           .etc
%                               .SegmentVector
%                               .SegmentLength
%                               .SegmentUnitVector
%                               .an
%                               .bn
%                               .cn
%
%
%                           .type       = 'CircularArk'
%                           .c          -- center
%                           .r          -- radius
%                           .anglerange -- a 1x2 vector of [startAngle, endAngle]
%                           .etc
%                               .endPoints -- 2x2 matrix

if nargin < 1
    indices = 1:UObstacles.N;
end
if ~isfield(World, 'N')
    World.N = 0;
end


for a = indices
    
    switch UObstacles.uobs{a}.Geometry.type
        
        case 'circle'
            
            World.N = World.N + 1;
            World.element{World.N}.type = 'CircularArk';
            World.element{World.N}.r = UObstacles.uobs{a}.Geometry.r;
            World.element{World.N}.c = UObstacles.uobs{a}.Geometry.c;
            World.element{World.N}.anglerange = [-pi, pi];
            World.Center(:,World.N) = World.element{World.N}.c;
            World.RMax(World.N) = World.element{World.N}.r;
            World.element{World.N}.etc.endPoints(:,1) = World.element{World.N}.c + ...
                World.element{World.N}.r*[cos(World.element{World.N}.anglerange(1)); sin(World.element{World.N}.anglerange(1))];
            World.element{World.N}.etc.endPoints(:,2) = World.element{World.N}.c + ...
                World.element{World.N}.r*[cos(World.element{World.N}.anglerange(2)); sin(World.element{World.N}.anglerange(2))];
            
            
        case 'straight_rectangle'
            xrange = UObstacles.uobs{a}.Geometry.xrange;
            yrange = UObstacles.uobs{a}.Geometry.yrange;
            x1indx = [1 1 2 2];
            x2indx = [1 2 2 1];
            y1indx = [1 2 2 1];
            y2indx = [2 2 1 1];
            for b = 1:4
                World.N = World.N + 1;
                World.element{World.N}.type = 'LineSegment';
                World.element{World.N}.ends = [xrange(x1indx(b)) xrange(x2indx(b)); yrange(y1indx(b)) yrange(y2indx(b))];
                World.Center(:,World.N) = mean(World.element{World.N}.ends')';
                World.RMax(World.N) = norm(World.element{World.N}.ends(:,2)-World.element{World.N}.ends(:,1))/2;
                World.element{World.N}.etc.SegmentVector = World.element{World.N}.ends(:,2)-World.element{World.N}.ends(:,1);
                World.element{World.N}.etc.SegmentLength = sqrt(sum(World.element{World.N}.etc.SegmentVector.^2));
                World.element{World.N}.etc.SegmentUnitVector = World.element{World.N}.etc.SegmentVector / World.element{World.N}.etc.SegmentLength;
                al = World.element{World.N}.ends(2,2) - World.element{World.N}.ends(2,1);
                bl = World.element{World.N}.ends(1,1) - World.element{World.N}.ends(1,2);
                cl = World.element{World.N}.ends(2,1)*World.element{World.N}.ends(1,2) - ...
                                                    World.element{World.N}.ends(1,1)*World.element{World.N}.ends(2,2);
                mag = sqrt(al*al + bl*bl);
                World.element{World.N}.etc.an = al / mag;
                World.element{World.N}.etc.bn = bl / mag;
                World.element{World.N}.etc.cn = cl / mag;
            end
%             World.N = World.N + 1;
%             World.element{World.N}.type = 'LineSegment';
%             World.element{World.N}.ends = [xrange(1) xrange(2); yrange(2) yrange(2)];
%             World.Center(:,World.N) = mean(World.element{World.N}.ends')';
%             World.RMax(World.N) = norm(World.element{World.N}.ends(:,2)-World.element{World.N}.ends(:,1))/2;
%             World.N = World.N + 1;
%             World.element{World.N}.type = 'LineSegment';
%             World.element{World.N}.ends = [xrange(2) xrange(2); yrange(2) yrange(1)];
%             World.Center(:,World.N) = mean(World.element{World.N}.ends')';
%             World.RMax(World.N) = norm(World.element{World.N}.ends(:,2)-World.element{World.N}.ends(:,1))/2;
%             World.N = World.N + 1;
%             World.element{World.N}.type = 'LineSegment';
%             World.element{World.N}.ends = [xrange(2) xrange(1); yrange(1) yrange(1)];
%             World.Center(:,World.N) = mean(World.element{World.N}.ends')';
%             World.RMax(World.N) = norm(World.element{World.N}.ends(:,2)-World.element{World.N}.ends(:,1))/2;
            
    end
    
end


% Adding the boundary lines
xrange = UBoundary(1,:);
yrange = UBoundary(2,:);
x1indx = [1 1 2 2];
x2indx = [1 2 2 1];
y1indx = [1 2 2 1];
y2indx = [2 2 1 1];
for b = 1:4
    World.N = World.N + 1;
    World.element{World.N}.type = 'LineSegment';
    World.element{World.N}.ends = [xrange(x1indx(b)) xrange(x2indx(b)); yrange(y1indx(b)) yrange(y2indx(b))];
    World.Center(:,World.N) = mean(World.element{World.N}.ends')';
    World.RMax(World.N) = norm(World.element{World.N}.ends(:,2)-World.element{World.N}.ends(:,1))/2;
    World.element{World.N}.etc.SegmentVector = World.element{World.N}.ends(:,2)-World.element{World.N}.ends(:,1);
    World.element{World.N}.etc.SegmentLength = sqrt(sum(World.element{World.N}.etc.SegmentVector.^2));
    World.element{World.N}.etc.SegmentUnitVector = World.element{World.N}.etc.SegmentVector / World.element{World.N}.etc.SegmentLength;
    al = World.element{World.N}.ends(2,2) - World.element{World.N}.ends(2,1);
    bl = World.element{World.N}.ends(1,1) - World.element{World.N}.ends(1,2);
    cl = World.element{World.N}.ends(2,1)*World.element{World.N}.ends(1,2) - ...
        World.element{World.N}.ends(1,1)*World.element{World.N}.ends(2,2);
    mag = sqrt(al*al + bl*bl);
    World.element{World.N}.etc.an = al / mag;
    World.element{World.N}.etc.bn = bl / mag;
    World.element{World.N}.etc.cn = cl / mag;
end
