function VGraph_AddObstacle_all
%{
******************************************************************************************
*                                                                                        *
*    A* search with homotopy class constraints                                           *
*    Copyright (C) 2010  Subhrajit Bhattacharya                                          *
*                                                                                        *
*    This program is free software: you can redistribute it and/or modify                *
*    it under the terms of the GNU General Public License as published by                *
*    the Free Software Foundation, either version 3 of the License, or                   *
*    (at your option) any later version.                                                 *
*                                                                                        *
*    This program is distributed in the hope that it will be useful,                     *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of                      *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                       *
*    GNU General Public License for more details <http://www.gnu.org/licenses/>.         *
*                                                                                        *
*                                                                                        *
*    Contact: subhrajit@gmail.com, http://fling.seas.upenn.edu/~subhrabh/                *
*                                                                                        *
*                                                                                        *
******************************************************************************************
%}


global VGraph Obstacles

% Structure Obstacles
%                    .N                -- number of obstacles
%                    .obs{i}           -- cell array of structure that defines i-th obstacle
%                            .shape    -- Can be 'circle' or 'polygon' (present code supports only polygons)
%                       polygon:
%                            .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%

% Structure VGraph
%                 .ActiveIndices        -- A vector containing the indices of the active clusters ( = [1,2,ObstaclesInSight+2])
%                 .VertexCluster{i}     -- All the vertices associated with cluster i
%                                           ( i=1 -> start position - the only dynamic cluster
%                                             i=2 -> goal position
%                                             i=p -> Obstacle number p-2 )
%                                  .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%
%                 .InterClusterEdges{i,j}  -- Edges joining clusters i and j [IMPORTANT: i <= j ]
%                                        .isComputed -- Indicates whether these edges have been computed before (Check using isfield)
%                                        .Edges -- A 6xE matrix of edges along with cost
%                                                                   = [ v_i1    v_i2    ...   ;
%                                                                       v_j1    v_j2    ...   ;
%                                                                       cost_1  cost_2  ...   ;
%                                                                       a_1     a_2     ...   ;
%                                                                       b_1     b_2     ...   ;
%                                                                       c_1     c_2     ...   ; ]
%                                                                   v_kp is the index of the vertex in VGraph.VertexCluster(k).Vertices
%
%                 .LeastCostPath(k) -- contains information about the k-th segment of the path
%                                  .i    -- Index of cluster from where the edge originates
%                                  .v_i  -- Vertex index of cluster i
%                                  .j    -- Index of cluster where the edge ends
%                                  .v_j  -- Vertex index of cluster j
%                                  .e    -- Edge index in VGraph.InterClusterEdges{i,j}.Edges
%
% c_k = norm( VGraph.VertexCluster{i}.Vertices(1:2,v_ik) - VGraph.VertexCluster{j}.Vertices(1:2,v_jk) )
%
global allEdgeInfos

VGraph.ActiveIndices = 1:(Obstacles.N+2);
for a=1:Obstacles.N
    VGraph.VertexCluster{a+2}.Vertices = [Obstacles.obs{a}.Vertices; ones([1 size(Obstacles.obs{a}.Vertices,2)])];
end
for a=1:(Obstacles.N+2)
    for b=1:(Obstacles.N+2)
        VGraph.InterClusterEdges{a,b}.Edges = [];
    end
end

allEdgeInfos = {};

for ClusterNo = 1:(Obstacles.N+2)
    % Adding self-edges
    selfEdgesExist = 0;
    if all(ClusterNo <= size(VGraph.VertexCluster))
        if isfield(VGraph.InterClusterEdges{ClusterNo,ClusterNo},'isComputed')
            selfEdgesExist = 1;
        end
    end

    VertexCountThisCluster = size(VGraph.VertexCluster{ClusterNo}.Vertices, 2);
    if VertexCountThisCluster<=1
        VGraph.InterClusterEdges{ClusterNo,ClusterNo}.Edges = [];
        continue;
    end
    if ~selfEdgesExist
        StartVertices = 1:VertexCountThisCluster;
        EndVertices = [2:VertexCountThisCluster, 1];
        EdgeValidity = VGraph.VertexCluster{ClusterNo}.Vertices(3,StartVertices) & VGraph.VertexCluster{ClusterNo}.Vertices(3,EndVertices);
        %StartVertices = StartVertices(EdgeValidity);
        %EndVertices = EndVertices(EdgeValidity);

        allAuxColumns = ComputeAuxiliaryEdgeColumn(VGraph.VertexCluster{ClusterNo}.Vertices(1:2,StartVertices), VGraph.VertexCluster{ClusterNo}.Vertices(1:2,EndVertices));
        VGraph.InterClusterEdges{ClusterNo,ClusterNo}.Edges = [ [ StartVertices ; EndVertices ; allAuxColumns ] , [ EndVertices ; StartVertices ; allAuxColumns ] ];
        %VGraph.InterClusterEdges{ClusterNo,ClusterNo}.Edges(3,:) = VGraph.InterClusterEdges{ClusterNo,ClusterNo}.Edges(3,:); % + not(EdgeValidity)*1e6;
        for q = 1:size(allAuxColumns,2)
            s.v = [ VGraph.VertexCluster{ClusterNo}.Vertices(1:2,StartVertices(q)), ...
                    VGraph.VertexCluster{ClusterNo}.Vertices(1:2,EndVertices(q)) ];
            s.a = allAuxColumns(2,q);
            s.b = allAuxColumns(3,q);
            s.c = allAuxColumns(4,q);
            allEdgeInfos{end+1} = s;
        end

        VGraph.InterClusterEdges{ClusterNo,ClusterNo}.isComputed = 1;
    end
end

% populate ALL edges
for a1=1:(Obstacles.N+2) % the cluster a1
    for b1=1:size(VGraph.VertexCluster{a1}.Vertices,2) % the verticex b1 (in cluster a1)
        
        for a2=(a1+1):(Obstacles.N+2) % the cluster a2 (a1<=a2)
            VGraph.InterClusterEdges{a1,a2}.isComputed = 1;
            for b2=1:size(VGraph.VertexCluster{a2}.Vertices,2) % the vertices b2 (in cluster a2)
                
                %if (b1>=b2)
                %    continue;
                %end
                
                Coord1 = VGraph.VertexCluster{a1}.Vertices(1:2,b1);
                Coord2 = VGraph.VertexCluster{a2}.Vertices(1:2,b2);
                AuxiliaryEdgeColumn = ComputeAuxiliaryEdgeColumn(Coord1, Coord2);
                
                s.v = [Coord1, Coord2];
                s.a = AuxiliaryEdgeColumn(2);
                s.b = AuxiliaryEdgeColumn(3);
                s.c = AuxiliaryEdgeColumn(4);
                cost = AuxiliaryEdgeColumn(1);
                
                isValid = true;
                for c=1:length(allEdgeInfos)
                    if isIntersect_Segments(s, allEdgeInfos{c}, 2)
                        isValid = false;
                        break;
                    end
                end
                if (isValid) % Add this edge
                    VGraph.InterClusterEdges{a1,a2}.Edges(:,end+1) = [b1  b2  cost  s.a  s.b  s.c]';
                    %VGraph.InterClusterEdges{a2,a1}.Edges(:,end+1) = [b2  b1  cost  s.a  s.b  s.c]';
                    % can put a dL-val here
                    %allEdgeInfos{end+1} = s;
%                 else
%                     disp(s.v);
%                     disp(allEdgeInfos{c}.v);
%                     pause;
                end
                
            end
        end
        
    end
end