function result = isIntersect_EdgeCluster(EdgeDetails, ClusterNo)
global VGraph Obstacles

% Structure EdgeDetails
%                      .i    -- Index of cluster from where the edge originates
%                      .v_i  -- Vertex index of cluster i
%                      .j    -- Index of cluster where the edge ends
%                      .v_j  -- Vertex index of cluster j
%                      .e    -- Edge index in VGraph.InterClusterEdges{i,j}.Edges
%

% Structure Obstacles
%                    .N                -- number of obstacles
%                    .obs{i}           -- cell array of structure that defines i-th obstacle
%                            .shape    -- Can be 'circle' or 'polygon' (present code supports only polygons)
%                       polygon:
%                            .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%

% Structure VGraph
%                 .ActiveIndices        -- A vector containing the indices of the active clusters ( = [1,2,ObstaclesInSight+2])
%                 .VertexCluster{i}     -- All the vertices associated with cluster i
%                                           ( i=1 -> start position - the only dynamic cluster
%                                             i=2 -> goal position
%                                             i=p -> Obstacle number p-2 )
%                                  .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%
%                 .InterClusterEdges{i,j}  -- Edges joining clusters i and j [IMPORTANT: i <= j ]
%                                        .isComputed -- Indicates whether these edges have been computed before (Check using isfield)
%                                        .Edges -- A 6xE matrix of edges along with cost
%                                                                   = [ v_i1    v_i2    ...   ;
%                                                                       v_j1    v_j2    ...   ;
%                                                                       cost_1  cost_2  ...   ;
%                                                                       a_1     a_2     ...   ;
%                                                                       b_1     b_2     ...   ;
%                                                                       c_1     c_2     ...   ; ]
%                                                                   v_kp is the index of the vertex in VGraph.VertexCluster(k).Vertices
%
%                 .LeastCostPath(k) -- contains information about the k-th segment of the path
%                                  .i    -- Index of cluster from where the edge originates
%                                  .v_i  -- Vertex index of cluster i
%                                  .j    -- Index of cluster where the edge ends
%                                  .v_j  -- Vertex index of cluster j
%                                  .e    -- Edge index in VGraph.InterClusterEdges{i,j}.Edges
%
% c_k = norm( VGraph.VertexCluster{i}.Vertices(1:2,v_ik) - VGraph.VertexCluster{j}.Vertices(1:2,v_jk) )
%

EdgeVector = VGraph.InterClusterEdges{EdgeDetails.i,EdgeDetails.j}.Edges(:,EdgeDetails.e);

% case 1 : all the points in cluster lie on same side of the edge - no intersection
CheckPoints = VGraph.VertexCluster{ClusterNo}.Vertices;
vertexSigns = sign( EdgeVector(4)*CheckPoints(1,:) + EdgeVector(5)*CheckPoints(2,:) + EdgeVector(6) );
min_vertexSigns = min(vertexSigns);
max_vertexSigns = max(vertexSigns);
if (min_vertexSigns==0 || min_vertexSigns==1) || (max_vertexSigns==0 || max_vertexSigns==-1)
    result = 0;
    return;
end

% case 2 : Check for intersection of individual line segments in the cluster with the edge
result = 0;
EdgeSegment.v = [ VGraph.VertexCluster{EdgeDetails.i}.Vertices(1:2,EdgeDetails.v_i), ...
                  VGraph.VertexCluster{EdgeDetails.j}.Vertices(1:2,EdgeDetails.v_j) ];
EdgeSegment.a = EdgeVector(4);
EdgeSegment.b = EdgeVector(5);
EdgeSegment.c = EdgeVector(6);
for a = 1:size(VGraph.InterClusterEdges{ClusterNo,ClusterNo}.Edges, 2)
    PolygonSideVector = VGraph.InterClusterEdges{ClusterNo,ClusterNo}.Edges(:,a);
    PolygonSide.v = [ VGraph.VertexCluster{ClusterNo}.Vertices(1:2,PolygonSideVector(1)), ...
                      VGraph.VertexCluster{ClusterNo}.Vertices(1:2,PolygonSideVector(2)) ];
    PolygonSide.a = PolygonSideVector(4);
    PolygonSide.b = PolygonSideVector(5);
    PolygonSide.c = PolygonSideVector(6);
    
    if isIntersect_Segments(EdgeSegment, PolygonSide)
        result = 1;
        break;
    end
end
