function [UpdatedUObstaclesIndices, CheckedUObstaclesIndices] = UpdateDynamicUObstacles(points)
global UObstacles

% UObstacles are uninflated obstacles - similar to Obstacles
% Obstacles need to be created from UObstacles

% UObstacles -- a structure defining various shapes
%           .N       -- number of obstacles
%           .uobs{a} -- cell array of obstacles
%
%                   .CreatedFromPoints -- boolian describing whether it is created from points
%                   .points -- (optional - required only if .CreatedFromPoints is 1) the coordinates of points
%                   .Geometry
%
%                            .type = 'circle'
%                            .r -- radius of circle
%                            .c -- center of circle
%
%                            .type = 'straight_rectangle'
%                            .xrange -- a vector [xMin, xMax]
%                            .yrange -- a vector [yMin, yMax]
%
%                            .type = 'polygon'
%                            .vertices -- 2xN matrix of coordinates
%

% 'points' is a 2xN matrix of coordinates of adjuscent points received from a LASER scan

SameObstaclePointsMaxSeparation = 0.1; % Should be relative
minBelongPercentage_DeterministicObstacle = 0.7;
minBelongPercentage_GeneratedObstacle = 0.4;

UpdatedUObstaclesIndices = [];
CheckedUObstaclesIndices = [];

pointsCount = size(points,2);
if pointsCount <= 1
    return;
end
xDiff = diff(points(1,:));
yDiff = diff(points(2,:));
adjDistsSq = xDiff.^2 + yDiff.^2;
breakPoints = [(adjDistsSq > SameObstaclePointsMaxSeparation), true];
endIndices = find(breakPoints);

startIndex = 1;
for endIndex = endIndices
    thesePoints = points(:, startIndex:endIndex);
    startIndex = endIndex + 1;
    if size(thesePoints,2) < 5
        continue;
    end
    boundaryCheck = isPointsOnBoundary(thesePoints, 0.8);
    if sum(boundaryCheck)/length(boundaryCheck) >= minBelongPercentage_DeterministicObstacle
        continue;
    else
        thesePoints = thesePoints(:,not(boundaryCheck));
    end
    
    KnownObstacle = 0;
    for a = 1:UObstacles.N
        belongPercentage = CheckPointsBelonging(thesePoints, UObstacles.uobs{a}, 0.2, UObstacles.uobs{a}.CreatedFromPoints); % ###
        if UObstacles.uobs{a}.CreatedFromPoints
            isBelong = (belongPercentage >= minBelongPercentage_GeneratedObstacle);
        else
            isBelong = (belongPercentage >= minBelongPercentage_DeterministicObstacle);
        end
        if isBelong
            if UObstacles.uobs{a}.CreatedFromPoints
                UObstacles.uobs{a}.points = [UObstacles.uobs{a}.points, thesePoints];
                isChanged = UpdateUObstacle(a); % ###
                if isChanged
                    UpdatedUObstaclesIndices = [UpdatedUObstaclesIndices, a];
                end
                CheckedUObstaclesIndices = [CheckedUObstaclesIndices, a];
            end
            KnownObstacle = 1;
            break;
        end
    end
    
    if ~KnownObstacle
        newObstacleIndex = length(UObstacles.uobs) + 1;
        UObstacles.uobs{newObstacleIndex}.CreatedFromPoints = 1;
        UObstacles.uobs{newObstacleIndex}.points = thesePoints;
        isChanged = UpdateUObstacle(newObstacleIndex); % ###
        if ~isChanged
            disp('WARNING: Something wrong in UpdateUObstacle!');
        end
        UObstacles.N = newObstacleIndex;
        UpdatedUObstaclesIndices = [UpdatedUObstaclesIndices, newObstacleIndex];
        CheckedUObstaclesIndices = [CheckedUObstaclesIndices, newObstacleIndex];
    end
    
end


% ===================================================================


function belongPercentage = CheckPointsBelonging(points, thisObs, allowance, additionalCheck)

Geometry = thisObs.Geometry;

if nargin < 3
    allowance = 0.2; % The allowance on points to deviate away from the boundaries of obstacles - Should be relative
end

pointsCount = size(points,2);

switch Geometry.type
    
    case 'circle'
        
        distsSq = sum( (points - repmat(Geometry.c, [1 pointsCount])) .^ 2 );
        belongTags = ((Geometry.r-allowance)^2 < distsSq) & (distsSq < (Geometry.r+allowance)^2 );
%         belongTags = (distsSq < (Geometry.r+allowance)^2 );
                    
    case 'straight_rectangle'
        
        xMinSideTags = ((Geometry.xrange(1)-allowance) < points(1,:)) & (points(1,:) < (Geometry.xrange(1)+allowance)) & ...
                        (Geometry.yrange(1)-allowance < points(2,:)) & (points(2,:) < Geometry.yrange(2)+allowance);
        xMaxSideTags = ((Geometry.xrange(2)-allowance) < points(1,:)) & (points(1,:) < (Geometry.xrange(2)+allowance)) & ...
                        (Geometry.yrange(1)-allowance < points(2,:)) & (points(2,:) < Geometry.yrange(2)+allowance);
        yMinSideTags = ((Geometry.yrange(1)-allowance) < points(2,:)) & (points(2,:) < (Geometry.yrange(1)+allowance)) & ...
                        (Geometry.xrange(1)-allowance < points(1,:)) & (points(1,:) < Geometry.xrange(2)+allowance);
        yMaxSideTags = ((Geometry.yrange(2)-allowance) < points(2,:)) & (points(2,:) < (Geometry.yrange(2)+allowance)) & ...
                        (Geometry.xrange(1)-allowance < points(1,:)) & (points(1,:) < Geometry.xrange(2)+allowance);
        belongTags = xMinSideTags | xMaxSideTags | yMinSideTags | yMaxSideTags;
%        belongTags = ((Geometry.xrange(1)-allowance) < points(1,:)) & (points(1,:) < (Geometry.xrange(2)+allowance)) & ...
%                      ((Geometry.yrange(1)-allowance) < points(2,:)) & (points(2,:) < (Geometry.yrange(2)+allowance));

end

belongPercentage = sum(belongTags) / length(belongTags);

if nargin > 3
    obsPoints = thisObs.points;
    if additionalCheck
        XMat1 = repmat(points(1,:), [length(obsPoints(1,:)), 1]);
        XMat2 = repmat(obsPoints(1,:)', [1, length(points(1,:))]);
        YMat1 = repmat(points(2,:), [length(obsPoints(2,:)), 1]);
        YMat2 = repmat(obsPoints(2,:)', [1, length(points(2,:))]);
        DistSqMat = (XMat1-XMat2).^2 + (YMat1-YMat2).^2;
        DistSqVec = DistSqMat(:);
        for p = 1:20
            [minDist(p), minIndex] = min(DistSqVec);
            DistSqVec = DistSqVec([1:minIndex-1, minIndex+1:end]);
        end
        if median(minDist) < 0.05 % 0.2
            belongPercentage = 1;
        end
    end
end





% ===================================================================


function isChanged = UpdateUObstacle(UObstacleNo)
global UObstacles
% Generates a gemetric shape for an obstacle from points

if ~UObstacles.uobs{UObstacleNo}.CreatedFromPoints
    isChanged = 0;
    return;
end
x = UObstacles.uobs{UObstacleNo}.points(1,:)';
y = UObstacles.uobs{UObstacleNo}.points(2,:)';
pointsCount = size(UObstacles.uobs{UObstacleNo}.points,2);

% -------------------------------

% Try fitting a circle
% Code snippet by Izhak bucher, 25/oct/1991, downloaded from
% http://www.mathworks.com/matlabcentral/fileexchange/loadFile.do?objectId=5557&objectType=File
Geometry{1}.type = 'circle';
a = [x y ones([pointsCount 1])]\[-(x.^2+y.^2)];
Geometry{1}.c = [ -a(1)/2; -a(2)/2 ];
Geometry{1}.r =  sqrt((a(1)^2+a(2)^2)/4-a(3));
tmp.Geometry = Geometry{1};
belongPercentage(1) = CheckPointsBelonging(UObstacles.uobs{UObstacleNo}.points, tmp, 0.3);
%%belongPercentage(1) = length(x)/Geometry{1}.r;
if length(x)/Geometry{1}.r < 20
    belongPercentage(1) = belongPercentage(1) * 0.5;
end
belongPercentage(1) = belongPercentage(1) * 0.75;


% Try fitting a straight_rectangle
sameEdgeMaxThickness = 0.02; % 2*allowance
Geometry{2}.type = 'straight_rectangle';
Geometry{2}.xrange = [min(x), max(x)];
Geometry{2}.yrange = [min(y), max(y)];
tmp.Geometry = Geometry{2};
belongPercentage(2) = CheckPointsBelonging(UObstacles.uobs{UObstacleNo}.points, tmp, 0.3);
%%belongPercentage(1) = length(x) / ( (Geometry{2}.xrange(2)-Geometry{2}.xrange(1)+Geometry{2}.yrange(2)-Geometry{2}.yrange(1)) / 4);

% -------------------------------

[minBelongPercentage, GeometryIndex] = max(belongPercentage);
%GeometryIndex = 1;
NewGeometry = Geometry{GeometryIndex};

if isfield(UObstacles.uobs{UObstacleNo}, 'Geometry')

    OldGeometry = UObstacles.uobs{UObstacleNo}.Geometry;
    eps = 0.2;

    % Assign value to isChanged
    if ~strcmp(OldGeometry.type, NewGeometry.type)
        isChanged = 1;
    else
        switch NewGeometry.type
            case 'circle'
                isChanged = (abs(OldGeometry.c(1)-NewGeometry.c(1)) > eps)  |  ...
                    (abs(OldGeometry.c(2)-NewGeometry.c(2)) > eps)  |  ...
                    (abs(OldGeometry.r-NewGeometry.r) > eps);
            case 'straight_rectangle'
                isChanged = (abs(OldGeometry.xrange(1)-NewGeometry.xrange(1)) > eps)  |  ...
                    (abs(OldGeometry.xrange(2)-NewGeometry.xrange(2)) > eps)  |  ...
                    (abs(OldGeometry.yrange(1)-NewGeometry.yrange(1)) > eps)  |  ...
                    (abs(OldGeometry.yrange(2)-NewGeometry.yrange(2)) > eps);
        end
    end

else
    isChanged = 1;
end

if isChanged
    UObstacles.uobs{UObstacleNo}.Geometry = NewGeometry;
end


% ===================================================================


function result = isPointsOnBoundary(points, allowance)
global Boundary
if nargin < 2
    allowance = 0.2; % The allowance on points to deviate away from the boundaries of obstacles - Should be relative
end

% xMinSideTags = ((Boundary(1,1)-allowance) < points(1,:)) & (points(1,:) < (Boundary(1,1)+allowance)) & ...
%     (Boundary(2,1) < points(2,:)) & (points(2,:) < Boundary(2,2));
% xMaxSideTags = ((Boundary(1,2)-allowance) < points(1,:)) & (points(1,:) < (Boundary(1,2)+allowance)) & ...
%     (Boundary(2,1) < points(2,:)) & (points(2,:) < Boundary(2,2));
% yMinSideTags = ((Boundary(2,1)-allowance) < points(2,:)) & (points(2,:) < (Boundary(2,1)+allowance)) & ...
%     (Boundary(1,1) < points(1,:)) & (points(1,:) < Boundary(1,2));
% yMaxSideTags = ((Boundary(2,2)-allowance) < points(2,:)) & (points(2,:) < (Boundary(2,2)+allowance)) & ...
%     (Boundary(1,1) < points(1,:)) & (points(1,:) < Boundary(1,2));

xMinSideTags = (points(1,:) < (Boundary(1,1)+allowance)) & ...
    (Boundary(2,1) < points(2,:)) & (points(2,:) < Boundary(2,2));
xMaxSideTags = ((Boundary(1,2)-allowance) < points(1,:)) & ...
    (Boundary(2,1) < points(2,:)) & (points(2,:) < Boundary(2,2));
yMinSideTags = (points(2,:) < (Boundary(2,1)+allowance)) & ...
    (Boundary(1,1) < points(1,:)) & (points(1,:) < Boundary(1,2));
yMaxSideTags = ((Boundary(2,2)-allowance) < points(2,:)) & ...
    (Boundary(1,1) < points(1,:)) & (points(1,:) < Boundary(1,2));

result = xMinSideTags | xMaxSideTags | yMinSideTags | yMaxSideTags;
