#include <assert.h>
#include <math.h>

#include <gazebo.h>
#include <gazebo/World.hh>
#include <gazebo/WorldFile.hh>
#include <gazebo/ModelFactory.hh>
#include <gazebo/RayGeom.hh>
#include <gazebo/BoxGeom.hh>
#include <gazebo/CylinderGeom.hh>
#include <gazebo/Body.hh>

#include <ctime>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

#include "Tracking_Noisy.hh"


//////////////////////////////////////////////////////////////////////////////
// Register this model
GZ_REGISTER_PLUGIN("Tracking_Noisy", Tracking_Noisy);


//////////////////////////////////////////////////////////////////////////////
// Constructor
Tracking_Noisy::Tracking_Noisy( World *world )
    : Model( world )
{
  this->body = NULL;

	rng = gsl_rng_alloc (gsl_rng_taus);
	gsl_rng_set(rng, time(0));


  return;
}


//////////////////////////////////////////////////////////////////////////////
// Destructor
Tracking_Noisy::~Tracking_Noisy()
{
  if (this->body)
    delete this->body;

  this->body = NULL;

	gsl_rng_free(rng);

  return;
}


//////////////////////////////////////////////////////////////////////////////
// Load the model
int Tracking_Noisy::Load( WorldFile *file, WorldFileNode *node )
{
  // Create a dummy body to which to attach geoms
  this->body = new Body(this->world, NULL, true);
  this->AddBody(this->body, true);

	this->cov_x = node->GetTupleDouble("noise", 0, 0.0);
	this->cov_y = node->GetTupleDouble("noise", 1, 0.0);
	this->cov_theta = node->GetTupleDouble("noise", 2, 0.0);

  return 0;
}


//////////////////////////////////////////////////////////////////////////////
// Initialize the model
int Tracking_Noisy::Init( WorldFile *file, WorldFileNode *node )
{ 
  // Initialize external interface
  if (this->IfaceInit() != 0)
    return -1;

  return 0;
}


//////////////////////////////////////////////////////////////////////////////
// Initialize the external interface
int Tracking_Noisy::IfaceInit()
{
  this->truth_iface = gz_truth_alloc();
  if (gz_truth_create( this->truth_iface, this->world->gz_server, this->GetId(),
                       "Tracking_Noisy", this->GetIntId(), this->GetParentIntId()) != 0)
    return -1;

  return 0;
}


//////////////////////////////////////////////////////////////////////////////
// Finalize the model
int Tracking_Noisy::Fini()
{
  this->IfaceFini();

  return 0;
}


//////////////////////////////////////////////////////////////////////////////
// Finalize the external interface
int Tracking_Noisy::IfaceFini()
{
  gz_truth_destroy( this->truth_iface );
  gz_truth_free( this->truth_iface );
  this->truth_iface = NULL;

  return 0;
}


//////////////////////////////////////////////////////////////////////////////
// Update the model
void Tracking_Noisy::Update( double /*step*/ )
{
  GzVector cmd_pos;
  GzQuatern cmd_rot;

  // Read commands from the truth interface
  if (IfaceGetCmd(&cmd_pos, &cmd_rot))
  {
    printf("Commanded Pos[%f %f %f] Rot[%f %f %f %f]\n",
        cmd_pos.x,cmd_pos.y,cmd_pos.z,cmd_rot.x,cmd_rot.y,cmd_rot.z,cmd_rot.u);

    if (!GzVectorFinite(cmd_pos))
      printf("Commanded Vector is not Finite\n");
    if (!GzQuaternFinite(cmd_rot))
      printf("Commanded Quatern is not Finite\n");

    // TODO: Not quite right; SetModelPose uses parent-relative
    // coordinates; should be world coordinates
    if (this->parent)
      this->world->SetModelPose(this->parent, GzPoseSet(cmd_pos, cmd_rot));
  }

  // Update the mmap interface with the new data
  IfacePutData();
  
  return;
}


//////////////////////////////////////////////////////////////////////////////
// Get commands from the interface
bool Tracking_Noisy::IfaceGetCmd(GzVector *pos, GzQuatern *rot)
{
  bool cmd_new;
  
  gz_truth_lock(this->truth_iface, 1);

  if (this->truth_iface->data->cmd_new)
  {
    cmd_new = true;
    this->truth_iface->data->cmd_new = 0;

    *pos = GzVectorSet(this->truth_iface->data->cmd_pos[0],
                       this->truth_iface->data->cmd_pos[1],
                       this->truth_iface->data->cmd_pos[2]);
    *rot = GzQuaternSet(this->truth_iface->data->cmd_rot[0],
                        this->truth_iface->data->cmd_rot[1],
                        this->truth_iface->data->cmd_rot[2],
                        this->truth_iface->data->cmd_rot[3]);

    if (rot->u==0 && rot->x==0 && rot->y==0 && rot->z==0)
      rot->u = 1.0;
  }
  else
    cmd_new = false;

  gz_truth_unlock(this->truth_iface);

  return cmd_new;
}


//////////////////////////////////////////////////////////////////////////////
// Update the data in the interface
void Tracking_Noisy::IfacePutData()
{
  GzPose pose;
	GzQuatern error_q;
	GzQuatern output_q;

  gz_truth_lock(this->truth_iface, 1);

  // Data timestamp
  this->truth_iface->data->time = this->world->GetSimTime();

  if (this->parent)
  {
    pose = this->parent->GetPose();

    this->truth_iface->data->pos[0] = pose.pos.x+gsl_ran_gaussian(rng, cov_x);
    this->truth_iface->data->pos[1] = pose.pos.y+gsl_ran_gaussian(rng, cov_y);
    this->truth_iface->data->pos[2] = pose.pos.z;

		error_q = GzQuaternFromEuler(0, 0, gsl_ran_gaussian(rng, cov_theta));
		output_q = GzQuaternMul(pose.rot, error_q);
																 
    this->truth_iface->data->rot[0] = output_q.u;
    this->truth_iface->data->rot[1] = output_q.x;
    this->truth_iface->data->rot[2] = output_q.y;
    this->truth_iface->data->rot[3] = output_q.z;
  }

  gz_truth_unlock(this->truth_iface);

  return;
}

